﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [Api(Description = "已结算未收费")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SettlementUnPayAnalysis : RestfulExecution<Settlement>
    {
        public Guid OwnerKvid { get; set; }
        public List<Guid> OwnerKvids { get; set; }
        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            var beginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            var endTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (endTime < beginTime) throw new Exception("查询结束日期不可小于开始日期!");
            if (endTime == beginTime) endTime = endTime.AddDays(1);
            if (OwnerKvids.IsNullOrEmpty()) OwnerKvids = new List<Guid>();
            if (OwnerKvid != Guid.Empty) OwnerKvids.Add(OwnerKvid);
            var conn = KiviiContext.GetOpenedDbConnection<Settlement>();
            var sqlExpress = conn.From<Settlement>();
            sqlExpress.Where(o => o.OffsetKvid == Guid.Empty);
            sqlExpress.And(o => o.CreateTime >= beginTime && o.CreateTime < endTime);
            //sqlExpress.And(o => o.AmountPayment < o.Amount || o.AmountPayment == 0);
            if (!OwnerKvids.IsNullOrEmpty()) sqlExpress.And(o => Sql.In(o.OwnerKvid, OwnerKvids));
            //if (OwnerKvid != Guid.Empty) sqlExpress.And(o => o.OwnerKvid == OwnerKvid);
            sqlExpress.Select(o => new { o.OwnerName, o.OwnerKvid, o.Type, o.AmountPayment, o.Amount });
            var results = conn.Select(sqlExpress);

            var rtns = new RestfulQueryResponse<StatisticResponse<Analysis>>();
            rtns.Results = new List<StatisticResponse<Analysis>>();
            if (results.IsNullOrEmpty()) return rtns;

            //待收费报告
            var rtnReports = new StatisticResponse<Analysis>();
            rtnReports.BeginTime = BeginTime;
            rtnReports.EndTime = EndTime;
            rtnReports.Total = results.Where(o=>o.Type== "Report").Count();
            rtnReports.TotalAmount = results.Where(o => o.Type == "Report").Sum(o => o.Amount);
            rtnReports.TotalAmountPayment = results.Where(o => o.Type == "Report").Sum(o => o.AmountPayment);
            rtnReports.TotalQuantityPayment = results.Where(o => o.Type == "Report").Count(o => o.AmountPayment == o.Amount);
            rtnReports.TotalAmountUnpaid = results.Where(o => o.Type == "Report").Sum(o => o.Amount - o.AmountPayment);
            rtnReports.TotalQuantityUnpaid = results.Where(o => o.Type == "Report").Count(o => o.AmountPayment == 0 || o.AmountPayment < o.Amount);
            rtns.Results.Add(rtnReports);

            //待收费政府任务
            var rtnSupervisions = new StatisticResponse<Analysis>();
            rtnSupervisions.BeginTime = BeginTime;
            rtnSupervisions.EndTime = EndTime;
            rtnSupervisions.Total = results.Where(o => o.Type == "Supervision").Count();
            rtnSupervisions.TotalAmount = results.Where(o => o.Type == "Supervision").Sum(o => o.Amount);
            rtnSupervisions.TotalAmountPayment = results.Where(o => o.Type == "Supervision").Sum(o => o.AmountPayment);
            rtnSupervisions.TotalQuantityPayment = results.Where(o => o.Type == "Supervision").Count(o => o.AmountPayment == o.Amount);
            rtnSupervisions.TotalAmountUnpaid = results.Where(o => o.Type == "Supervision").Sum(o => o.Amount - o.AmountPayment);
            rtnSupervisions.TotalQuantityUnpaid = results.Where(o => o.Type == "Supervision").Count(o => o.AmountPayment == 0 || o.AmountPayment < o.Amount);
            rtns.Results.Add(rtnSupervisions);

            //待收费合同
            var rtnServices = new StatisticResponse<Analysis>();
            rtnServices.BeginTime = BeginTime;
            rtnServices.EndTime = EndTime;
            rtnServices.Total = results.Where(o => o.Type == "Service").Count();
            rtnServices.TotalAmount = results.Where(o => o.Type == "Service").Sum(o => o.Amount);
            rtnServices.TotalAmountPayment = results.Where(o => o.Type == "Service").Sum(o => o.AmountPayment);
            rtnServices.TotalQuantityPayment = results.Where(o => o.Type == "Service").Count(o => o.AmountPayment == o.Amount);
            rtnServices.TotalAmountUnpaid = results.Where(o => o.Type == "Service").Sum(o => o.Amount - o.AmountPayment);
            rtnServices.TotalQuantityUnpaid = results.Where(o => o.Type == "Service").Count(o => o.AmountPayment == 0 || o.AmountPayment < o.Amount);
            rtns.Results.Add(rtnServices);

            //待收费证书
            var rtnCertificates = new StatisticResponse<Analysis>();
            rtnCertificates.BeginTime = BeginTime;
            rtnCertificates.EndTime = EndTime;
            rtnCertificates.Total = results.Where(o => o.Type == "Certificate").Count();
            rtnCertificates.TotalAmount = results.Where(o => o.Type == "Certificate").Sum(o => o.Amount);
            rtnCertificates.TotalAmountPayment = results.Where(o => o.Type == "Certificate").Sum(o => o.AmountPayment);
            rtnCertificates.TotalQuantityPayment = results.Where(o => o.Type == "Certificate").Count(o => o.AmountPayment == o.Amount);
            rtnCertificates.TotalAmountUnpaid = results.Where(o => o.Type == "Certificate").Sum(o => o.Amount - o.AmountPayment);
            rtnCertificates.TotalQuantityUnpaid = results.Where(o => o.Type == "Certificate").Count(o => o.AmountPayment == 0 || o.AmountPayment < o.Amount);
            rtns.Results.Add(rtnCertificates);

            //待收费挂签
            var rtnTags = new StatisticResponse<Analysis>();
            rtnTags.BeginTime = BeginTime;
            rtnTags.EndTime = EndTime;
            rtnTags.Total = results.Where(o => o.Type == "Tag").Count();
            rtnTags.TotalAmount = results.Where(o => o.Type == "Tag").Sum(o => o.Amount);
            rtnTags.TotalAmountPayment = results.Where(o => o.Type == "Tag").Sum(o => o.AmountPayment);
            rtnTags.TotalQuantityPayment = results.Where(o => o.Type == "Tag").Count(o => o.AmountPayment == o.Amount);
            rtnTags.TotalAmountUnpaid = results.Where(o => o.Type == "Tag").Sum(o => o.Amount - o.AmountPayment);
            rtnTags.TotalQuantityUnpaid = results.Where(o => o.Type == "Tag").Count(o => o.AmountPayment == 0 || o.AmountPayment < o.Amount);
            rtns.Results.Add(rtnTags);

            //待收费其他
            var rtnOthers = new StatisticResponse<Analysis>();
            rtnOthers.BeginTime = BeginTime;
            rtnOthers.EndTime = EndTime;
            rtnOthers.Total = results.Where(o => o.Type != "Report" && o.Type != "Supervision" && o.Type != "Service" && o.Type != "Certificate" && o.Type != "Tag").Count();
            rtnOthers.TotalAmount = results.Where(o => o.Type != "Report" && o.Type != "Supervision" && o.Type != "Service" && o.Type != "Certificate" && o.Type != "Tag").Sum(o => o.Amount);
            rtnOthers.TotalAmountPayment = results.Where(o => o.Type != "Report" && o.Type != "Supervision" && o.Type != "Service" && o.Type != "Certificate" && o.Type != "Tag").Sum(o => o.AmountPayment);
            rtnOthers.TotalQuantityPayment = results.Where(o => o.Type != "Report" && o.Type != "Supervision" && o.Type != "Service" && o.Type != "Certificate" && o.Type != "Tag").Count(o => o.AmountPayment == o.Amount);
            rtnOthers.TotalAmountUnpaid = results.Where(o => o.Type != "Report" && o.Type != "Supervision" && o.Type != "Service" && o.Type != "Certificate" && o.Type != "Tag").Sum(o => o.Amount - o.AmountPayment);
            rtnOthers.TotalQuantityUnpaid = results.Where(o => o.Type != "Report" && o.Type != "Supervision" && o.Type != "Service" && o.Type != "Certificate" && o.Type != "Tag").Count(o => o.AmountPayment == 0 || o.AmountPayment < o.Amount);
            rtns.Results.Add(rtnOthers);

            return rtns;
        }
    }
}
